@@ -1,9 +1,10 @@
 package Dancer2::Plugin::Auth::Extensible::Provider::Example;
 
-use strict;
-use base "Dancer2::Plugin::Auth::Extensible::Provider::Base";
+use Moo;
+with "Dancer2::Plugin::Auth::Extensible::Role::Provider";
+use namespace::clean;
 
-our $VERSION = '0.502';
+our $VERSION = '0.600';
 
 # A more sensible provider would be likely to get this information from e.g. a
 # database (or LDAP, or...) rather than hardcoding it.  This, however, is an
