@@ -1,10 +1,12 @@
 package Dancer2::Plugin::Auth::Extensible::Provider::Unix;
 
-use strict;
-use base 'Dancer2::Plugin::Auth::Extensible::Provider::Base';
 use Authen::Simple::PAM;
+use Unix::Passwd::File;
+use Moo;
+with "Dancer2::Plugin::Auth::Extensible::Role::Provider";
+use namespace::clean;
 
-our $VERSION = '0.502';
+our $VERSION = '0.600';
 
 =head1 NAME
 
@@ -21,6 +23,14 @@ and L<Authen::Simple::PAM> to perform authentication via PAM.
 Unix group membership is used as a reasonable facsimile for roles - this seems
 sensible.
 
+B<WARNING>: in order to use PAM authentication on most modern Linux/UNIX
+systems the application performing authentication must have read access
+to the C</etc/shadow> file. B<This is a security risk> since it can lead
+to accidental disclosure of sensitive data if you have any path traversal
+vulnerabilities, etc. We strongly recommend B<AGAINST> using this module
+and provide it purely as an example. Any use of it B<IS AT YOUR OWN RISK>.
+You have been warned.
+
 =head1 METHODS
 
 =head2 authenticate_user $username, $password
@@ -36,7 +46,7 @@ sub authenticate_user {
 =head2 get_user_details $username
 
 Returns information from the C<passwd> file as a hash reference with the
-following keys: uid, gid, quota, comment, gcos,  dir, shell, expire 
+following keys: uid, gid, quota, comment, gecos,  dir, shell, expire 
 
 =cut
 
@@ -52,7 +62,7 @@ sub get_user_details {
         gid      => $result[3],
         quota    => $result[4],
         comment  => $result[5],
-        gcos     => $result[6],
+        gecos    => $result[6],
         dir      => $result[7],
         shell    => $result[8],
         expire   => $result[9],
