@@ -1,6 +1,6 @@
 package Dancer2::Plugin::Auth::Extensible::Test;
 
-our $VERSION = '0.502';
+our $VERSION = '0.600';
 
 =head1 NAME
 
@@ -69,8 +69,8 @@ sub testme {
     test_psgi $app, _test_base();
 
     foreach my $name (
-        qw/ create_user update_user password_reset user_password
-        lastlogin expired/
+        qw/ create_user update_user update_roles password_reset user_password
+        lastlogin expired reset_code/
       )
     {
         if ( delete $args{$name} ) {
@@ -344,8 +344,12 @@ sub _test_base {
 
             my $res = $cb->(POST '/logout', @headers );
 
-            is($res->code, 200, 'Logging out returns 200')
+            is($res->code, 302, 'Logging out returns 302')
                 or diag explain $trap->read;
+
+            is($res->headers->header('Location'),
+               'http://localhost/',
+               '/logout redirected to / (exit_page) after logging out');
         }
 
         # Check we can't access protected pages now we logged out:
@@ -435,19 +439,6 @@ sub _test_base {
                 'Correct roles for other user in current realm' );
         }
 
-        # check roles: this user has no roles
-
-        {
-            $trap->read; # clear logs
-
-            my $res = $cb->( GET '/roles', @headers );
-
-            is($res->code, 200, 'get /roles is 200 - this user has no roles')
-                or diag explain $trap->read;
-
-            is ($res->content, '', 'Correct roles for logged in user');
-        }
-
         # Now, log out again
 
         {
@@ -455,8 +446,12 @@ sub _test_base {
 
             my $res = $cb->(POST '/logout', @headers );
 
-            is($res->code, 200, 'Logged out again')
+            is($res->code, 302, 'Logging out returns 302')
                 or diag explain $trap->read;
+
+            is($res->headers->header('Location'),
+               'http://localhost/',
+               '/logout redirected to / (exit_page) after logging out');
         }
 
         # Now check we can log in as a user whose password is stored hashed:
@@ -531,8 +526,12 @@ sub _test_base {
             $trap->read; # clear logs
 
             my $res = $cb->(POST '/logout', @headers );
-            is $res->code, 200, 'Logged out again'
+            is($res->code, 302, 'Logging out returns 302')
                 or diag explain $trap->read;
+
+            is($res->headers->header('Location'),
+               'http://localhost/',
+               '/logout redirected to / (exit_page) after logging out');
         }
 
         # require_login should receive a coderef
@@ -742,9 +741,7 @@ sub _test_create_user {
 };
 
 sub _test_no_create_user {
-
-    note "test no create_user";
-
+    note "NOTE: not testing create_user";
     my $sub = sub {
         my $trap = TestApp->dancer_app->logger_engine->trapper;
         my $cb = shift;
@@ -754,6 +751,9 @@ sub _test_no_create_user {
 # update_user
 
 sub _test_update_user {
+
+    note "test update_user";
+
     my $sub = sub {
         my $trap = TestApp->dancer_app->logger_engine->trapper;
         my $cb = shift;
@@ -801,6 +801,32 @@ sub _test_update_user {
                     "Name is now Wiltshire Apples $realm"
                 );
             }
+        }
+    }
+};
+
+sub _test_no_update_user {
+    note "NOTE: not testing update_user";
+    my $sub = sub {
+        my $trap = TestApp->dancer_app->logger_engine->trapper;
+        my $cb = shift;
+    }
+};
+
+
+# update_roles
+# This is pretty much DBIC provider at the moment until D2PAE itself defines
+# how role changes can be performed.
+
+sub _test_update_roles {
+
+    note "test update_user";
+
+    my $sub = sub {
+        my $trap = TestApp->dancer_app->logger_engine->trapper;
+        my $cb = shift;
+
+        for my $realm (qw/config1 config2/) {
 
             # Now we're going to update the current user and add a role
 
@@ -852,12 +878,12 @@ sub _test_update_user {
                 is( $res->code, 302,
                     "[GET /cider] Correct code for realm $realm" );
 
-                $trap->read; # clear logs
+                diag explain $trap->read; # clear logs
 
                 # Now add the role
                 $res = $cb->( GET "/update_user_role/$realm" );
 
-                $trap->read; # clear logs
+                diag explain $trap->read; # clear logs
 
                 # And see whether we're now allowed access
                 $res = $cb->( GET '/cider', @headers );
@@ -871,7 +897,8 @@ sub _test_update_user {
     }
 };
 
-sub _test_no_update_user {
+sub _test_no_update_roles {
+    note "NOTE: not testing update_roles";
     my $sub = sub {
         my $trap = TestApp->dancer_app->logger_engine->trapper;
         my $cb = shift;
@@ -881,6 +908,7 @@ sub _test_no_update_user {
 # password_reset
 
 sub _test_password_reset {
+    die "FIXME: no password_reset tests";
     my $sub = sub {
         my $trap = TestApp->dancer_app->logger_engine->trapper;
         my $cb = shift;
@@ -888,6 +916,7 @@ sub _test_password_reset {
 };
 
 sub _test_no_password_reset {
+    note "NOTE: not testing password_reset";
     my $sub = sub {
         my $trap = TestApp->dancer_app->logger_engine->trapper;
         my $cb = shift;
@@ -897,6 +926,7 @@ sub _test_no_password_reset {
 # user_password
 
 sub _test_user_password {
+    die "FIXME: no user_password tests";
     my $sub = sub {
         my $trap = TestApp->dancer_app->logger_engine->trapper;
         my $cb = shift;
@@ -904,6 +934,7 @@ sub _test_user_password {
 };
 
 sub _test_no_user_password {
+    note "NOTE: not testing user_password";
     my $sub = sub {
         my $trap = TestApp->dancer_app->logger_engine->trapper;
         my $cb = shift;
@@ -913,6 +944,7 @@ sub _test_no_user_password {
 # lastlogin
 
 sub _test_lastlogin {
+    die "FIXME: no lastlogin tests";
     my $sub = sub {
         my $trap = TestApp->dancer_app->logger_engine->trapper;
         my $cb = shift;
@@ -920,6 +952,7 @@ sub _test_lastlogin {
 };
 
 sub _test_no_lastlogin {
+    note "NOTE: not testing lastlogin";
     my $sub = sub {
         my $trap = TestApp->dancer_app->logger_engine->trapper;
         my $cb = shift;
@@ -929,6 +962,7 @@ sub _test_no_lastlogin {
 # expired
 
 sub _test_expired {
+    die "FIXME: no expired tests";
     my $sub = sub {
         my $trap = TestApp->dancer_app->logger_engine->trapper;
         my $cb = shift;
@@ -936,14 +970,19 @@ sub _test_expired {
 };
 
 sub _test_no_expired {
+    note "NOTE: not testing expired";
     my $sub = sub {
         my $trap = TestApp->dancer_app->logger_engine->trapper;
         my $cb = shift;
     }
 };
 
-#########
+# reset code
+
 sub _test_reset_code {
+
+    note "test reset_code";
+
     my $sub = sub {
         my $trap = TestApp->dancer_app->logger_engine->trapper;
         my $cb = shift;
@@ -969,8 +1008,8 @@ sub _test_reset_code {
             my $logs = $trap->read;
             is $logs->[0]->{level}, 'debug', "we got a debug log message";
             like $logs->[0]->{message},
-              qr/^Failed to check for code with config.+fooget_user_by_code/,
-              "message is: Failed to check for code with config...";
+              qr/^No user found in realm config\d with code beer$/,
+              "message is: No user found in realm configX with code beer";
         }
         {
             $trap->read; # clear logs
@@ -981,8 +1020,8 @@ sub _test_reset_code {
             my $logs = $trap->read;
             is $logs->[0]->{level}, 'error', "we got a debug log message";
             like $logs->[0]->{message},
-              qr/^Route exception: set_user_password was not implemented/,
-              "message is: 'Route exception: set_user_password was not implemented...'";
+              qr/^Route exception: No username specified and no logged-in user/,
+              "message is: 'Route exception: No username specified and no logged-in user'";
         }
         {
             $trap->read; # clear logs
@@ -999,4 +1038,12 @@ sub _test_reset_code {
     }
 };
 
+sub _test_no_reset_code {
+    note "NOTE: not testing reset_code";
+    my $sub = sub {
+        my $trap = TestApp->dancer_app->logger_engine->trapper;
+        my $cb = shift;
+    }
+};
+
 1;
